==> gridstack-8.x-2.10.tar.gz: gridstack/src/Plugin/views/style/GridStackViews.php <==
<?php

namespace Drupal\gridstack\Plugin\views\style;

use Drupal\Core\Form\FormStateInterface;
use Drupal\blazy\Views\BlazyStylePluginBase;
use Drupal\gridstack\GridStackDefault as Defaults;
use Drupal\gridstack\Entity\GridStack;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * GridStack style plugin.
 */
class GridStackViews extends BlazyStylePluginBase {

  /**
   * The gridstack service manager.
   *
   * @var \Drupal\gridstack\GridStackManagerInterface
   */
  protected $manager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->manager = $container->get('gridstack.manager');
    $instance->blazyManager = $instance->blazyManager ?? $container->get('blazy.manager');

    return $instance;
  }

  /**
   * Returns the gridstack manager.
   */
  public function manager() {
    return $this->manager;
  }

  /**
   * Returns the gridstack admin.
   */
  public function admin() {
    return \Drupal::service('gridstack.admin');
  }

  /**
   * Overrides parent::buildOptionsForm().
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    $definition = $this->getDefinedFormScopes();
    $this->buildSettingsForm($form, $definition);
  }

  /**
   * Overrides StylePluginBase::render().
   */
  public function render() {
    $settings  = $this->buildSettings();
    $optionset = GridStack::loadWithFallback($settings['optionset']);
    $elements  = [];

    foreach ($this->renderGrouping($this->view->result, $settings['grouping']) as $rows) {
      $settings = array_filter($settings, function ($value) {
        return ($value !== NULL && $value !== '' && $value !== []);
      });

      $items = $this->buildElements($settings, $rows);

      // Supports Blazy multi-breakpoint images if using Blazy formatter.
      $blazies = $settings['blazies'] ?? NULL;
      if ($data = $this->getFirstImage($rows[0] ?? NULL)) {
        if ($blazies) {
          $blazies->set('first.data', $data);
        }
        // @todo remove post Blazy 2.10.
        else {
          $settings['first_image'] = $data;
        }
      }
      $build = [
        'items'     => $items,
        'optionset' => $optionset,
        'settings'  => $settings,
      ];

      $elements = $this->manager->build($build);
      unset($build);
    }

    return $elements;
  }

  /**
   * Returns gridstack contents.
   */
  public function buildElements(array $settings, $rows) {
    $build = [];
    $blazies = $settings['blazies'];
    $item_id = $blazies->get('item.id');
    $stamp_index = $settings['stamp_index'] ?? -1;

    foreach ($rows as $delta => $row) {
      $this->view->row_index = $delta;

      $box = $box['attributes'] = [];
      $settings['delta'] = $delta;

      // Adds box type where image can be anything, not only image.
      if (!empty($settings['image'])
        && $this->getField($delta, $settings['image'])) {
        $settings['type'] = $row->_entity->hasField($settings['image'])
        ? $row->_entity
          ->getFieldDefinition($settings['image'])
          ->getFieldStorageDefinition()
          ->getType()
        : '';
      }

      $blazies->reset($settings);
      $blazy = $settings['blazies'];
      $blazy->set('delta', $delta);

      $box['settings'] = $settings;

      // Use Vanilla gridstack if so configured, ignoring GridStack markups.
      if (!empty($settings['vanilla'])) {
        $box[$item_id] = $this->view->rowPlugin->render($row);
      }
      else {
        // Build individual row/element contents.
        $this->buildElement($box, $row, $delta);
      }

      // Allows extending contents without overriding the entire loop method.
      $this->buildElementExtra($box, $row, $delta);

      // Extracts stamp from existing box.
      $stamp = [];
      if ($delta == $stamp_index && !empty($box['stamp'])) {
        $stamp = $box['stamp'];
        $stamp[$stamp_index]['settings']['type'] = 'stamp';
        unset($box['stamp']);
      }

      // Build gridstack items.
      $build[] = $box;

      // Inserts stamp to the array.
      if ($delta == $stamp_index && !empty($stamp)) {
        array_splice($build, $stamp_index, 0, $stamp);
      }

      unset($box);
    }

    unset($this->view->row_index);
    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function buildElement(array &$box, $row, $delta) {
    $settings = &$box['settings'];
    $blazies = $settings['blazies'];
    $item_id = $blazies->get('item.id');
    $use_category = TRUE;

    // Add row/ item classes.
    if (!empty($settings['class'])) {
      $classes = $this->getFieldString($row, $settings['class'], $delta);
      if (!empty($classes[$delta])) {
        $box['attributes']['class'][] = $classes[$delta];
      }
    }

    // Adds the rich box named overlay replacing the main stage to alternate
    // boring images with a mix of rich boxes: Slick carousel, any potential
    // block_field: video, currency, time, weather, ads, donations blocks, etc.
    if (!empty($settings['overlay']) && $this->getField($delta, $settings['overlay'])) {
      $box[$item_id] = $this->getFieldRendered($delta, $settings['overlay']);
      $settings['type'] = 'rich';
      $use_category = FALSE;

      // As this takes over the entire box contents, nullify extra unused works.
      foreach (['caption', 'image', 'link', 'overlay', 'title'] as $key) {
        $settings[$key] = '';
      }
    }

    // Adds stamp, such as HTML list of latest news, members, testimonials, etc.
    $stamp_index = $settings['stamp_index'] ?? -1;
    if ($delta == $stamp_index && !empty($settings['stamp'])) {
      [$view_id, $display_id] = explode(":", $settings['stamp'], 2);

      $box['stamp'][$stamp_index] = [
        $item_id => views_embed_view($view_id, $display_id),
        'settings' => $settings,
        'attributes' => $box['attributes'] ?? [],
      ];
    }

    // Overrides parent::buildElement().
    parent::buildElement($box, $row, $delta);

    // Adds category, must run after parent::buildElement() where captions may
    // be nullified.
    if ($use_category && !empty($settings['category']) && $this->getField($delta, $settings['category'])) {
      $box['caption']['category'] = $this->getFieldRendered($delta, $settings['category']);
    }
  }

  /**
   * Returns extra row/ element content such as Isotope filters, sorters, etc.
   */
  public function buildElementExtra(array &$box, $row, $delta) {
    // Do nothing, let extender do their jobs.
  }

  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = ['stamp' => ['default' => '']];
    foreach (Defaults::extendedSettings() as $key => $value) {
      $options[$key] = ['default' => $value];
    }
    return $options + parent::defineOptions();
  }

  /**
   * Returns the defined scopes for the current form.
   */
  protected function getDefinedFormScopes(array $extra_fields = []) {
    // Pass the common field options relevant to this style.
    $fields = Defaults::viewFieldOptions();
    $fields = array_merge($fields, $extra_fields);

    // Fetches the returned field definitions to be used to define form scopes.
    $definition = $this->getDefinedFieldOptions($fields);
    $definition['opening_class'] = 'form--views';
    $definition['_views'] = TRUE;

    return $definition;
  }

  /**
   * {@inheritdoc}
   */
  protected function buildSettings() {
    $settings  = parent::buildSettings();
    $settings += Defaults::htmlSettings();
    $blazies   = $settings['blazies'];

    $blazies->set('namespace', 'gridstack')
      ->set('item.id', 'box')
      ->set('is.noratio', TRUE);

    // Prepare needed settings to work with.
    $settings['caption'] = array_filter($settings['caption']);
    $settings['ratio'] = '';

    return $settings;
  }

  /**
   * Build the Gridstack settings form.
   */
  protected function buildSettingsForm(&$form, &$definition) {
    $count = empty($definition['captions']) ? 0 : count($definition['captions']);
    $definition['captions_count'] = $count;
    $definition['stamps'] = $this->getViewsAsOptions('html_list');

    $this->admin()->buildSettingsForm($form, $definition);

    $title = '<p class="form__header form__title">';
    $title .= $this->t('Check Vanilla if using content/custom markups, not fields. <small>See it under <strong>Format > Show</strong> section. Otherwise Gridstack markups apply which require some fields added below.</small>');
    $title .= '</p>';

    $form['opening']['#markup'] .= $title;
    $form['image']['#description'] .= ' ' . $this->t('Be sure to UNCHECK "Use field template" (by default already UNCHECKED) to have it work for Blazy lazyloading. Use Blazy formatters for relevant features such as Colorbox/Photobox/Photoswipe, etc., or multimedia supports.');
  }

}

==> gridstack-8.x-2.11.tar.gz: gridstack/src/Plugin/views/style/GridStackViews.php <==
<?php

namespace Drupal\gridstack\Plugin\views\style;

use Drupal\Core\Form\FormStateInterface;
use Drupal\blazy\Views\BlazyStylePluginBase;
use Drupal\gridstack\GridStackDefault as Defaults;
use Drupal\gridstack\Entity\GridStack;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * GridStack style plugin.
 */
class GridStackViews extends BlazyStylePluginBase implements GridStackViewsInterface {

  /**
   * The gridstack service manager.
   *
   * @var \Drupal\gridstack\GridStackManagerInterface
   */
  protected $manager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->manager = $container->get('gridstack.manager');
    $instance->blazyManager = $instance->blazyManager ?? $container->get('blazy.manager');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function manager() {
    return $this->manager;
  }

  /**
   * {@inheritdoc}
   */
  public function admin() {
    return \Drupal::service('gridstack.admin');
  }

  /**
   * Overrides parent::buildOptionsForm().
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    $definition = $this->getDefinedFormScopes();
    $this->buildSettingsForm($form, $definition);
  }

  /**
   * Overrides StylePluginBase::render().
   */
  public function render() {
    $settings  = $this->buildSettings();
    $optionset = GridStack::loadWithFallback($settings['optionset']);
    $elements  = [];

    foreach ($this->renderGrouping($this->view->result, $settings['grouping']) as $rows) {
      $settings = array_filter($settings, function ($value) {
        return ($value !== NULL && $value !== '' && $value !== []);
      });

      $items = $this->buildElements($settings, $rows);

      // Supports Blazy multi-breakpoint images if using Blazy formatter.
      $blazies = $settings['blazies'];
      if ($data = $this->getFirstImage($rows[0] ?? NULL)) {
        $blazies->set('first.data', $data);
      }
      $build = [
        'items'     => $items,
        'optionset' => $optionset,
        'settings'  => $settings,
      ];

      $elements = $this->manager->build($build);
      unset($build);
    }

    return $elements;
  }

  /**
   * Returns gridstack contents.
   */
  public function buildElements(array $settings, $rows) {
    $build = [];
    $blazies = $settings['blazies'];
    $item_id = $blazies->get('item.id');
    $stamp_index = $settings['stamp_index'] ?? -1;
    $_image = $settings['image'] ?? NULL;

    foreach ($rows as $delta => $row) {
      $this->view->row_index = $delta;

      $box = $box['attributes'] = [];

      // Adds box type where image can be anything, not only image.
      if ($_image && $this->getField($delta, $_image)) {
        $settings['type'] = $row->_entity->hasField($_image)
          ? $row->_entity
            ->getFieldDefinition($_image)
            ->getFieldStorageDefinition()
            ->getType()
          : '';
      }

      $settings['delta'] = $delta;
      $blazies->reset($settings);
      $blazy = $settings['blazies'];
      $blazy->set('delta', $delta);

      $box['settings'] = $settings;

      // Use Vanilla gridstack if so configured, ignoring GridStack markups.
      if (!empty($settings['vanilla'])) {
        $box[$item_id] = $this->view->rowPlugin->render($row);
      }
      else {
        // Build individual row/element contents.
        $this->buildElement($box, $row, $delta);
      }

      // Allows extending contents without overriding the entire loop method.
      $this->buildElementExtra($box, $row, $delta);

      // Extracts stamp from existing box.
      $stamp = [];
      if ($delta == $stamp_index && !empty($box['stamp'])) {
        $stamp = $box['stamp'];
        $stamp[$stamp_index]['settings']['type'] = 'stamp';
        unset($box['stamp']);
      }

      // Build gridstack items.
      $build[] = $box;

      // Inserts stamp to the array.
      if ($delta == $stamp_index && !empty($stamp)) {
        array_splice($build, $stamp_index, 0, $stamp);
      }

      unset($box);
    }

    unset($this->view->row_index);
    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function buildElement(array &$box, $row, $delta) {
    $settings = &$box['settings'];
    $blazies = $settings['blazies'];
    $item_id = $blazies->get('item.id');
    $use_category = TRUE;

    // Add row/ item classes.
    if (!empty($settings['class'])) {
      $classes = $this->getFieldString($row, $settings['class'], $delta);
      if (!empty($classes[$delta])) {
        $box['attributes']['class'][] = $classes[$delta];
      }
    }

    // Adds the rich box named overlay replacing the main stage to alternate
    // boring images with a mix of rich boxes: Slick carousel, any potential
    // block_field: video, currency, time, weather, ads, donations blocks, etc.
    if (!empty($settings['overlay'])
      && $this->getField($delta, $settings['overlay'])) {
      $box[$item_id] = $this->getFieldRendered($delta, $settings['overlay']);
      $settings['type'] = 'rich';
      $use_category = FALSE;

      // As this takes over the entire box contents, nullify extra unused works.
      foreach (['caption', 'image', 'link', 'overlay', 'title'] as $key) {
        $settings[$key] = '';
      }
    }

    // Adds stamp, such as HTML list of latest news, members, testimonials, etc.
    $stamp_index = $settings['stamp_index'] ?? -1;
    if ($delta == $stamp_index && !empty($settings['stamp'])) {
      [$view_id, $display_id] = explode(":", $settings['stamp'], 2);

      $box['stamp'][$stamp_index] = [
        $item_id => views_embed_view($view_id, $display_id),
        'settings' => $settings,
        'attributes' => $box['attributes'] ?? [],
      ];
    }

    // Overrides parent::buildElement().
    parent::buildElement($box, $row, $delta);

    // Adds category, must run after parent::buildElement() where captions may
    // be nullified.
    if ($use_category
      && !empty($settings['category'])
      && $this->getField($delta, $settings['category'])) {
      $box['caption']['category'] = $this->getFieldRendered($delta, $settings['category']);
    }
  }

  /**
   * Returns extra row/ element content such as Isotope filters, sorters, etc.
   */
  public function buildElementExtra(array &$box, $row, $delta) {
    // Do nothing, let extender do their jobs.
  }

  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = ['stamp' => ['default' => '']];
    foreach (Defaults::extendedSettings() as $key => $value) {
      $options[$key] = ['default' => $value];
    }
    return $options + parent::defineOptions();
  }

  /**
   * Returns the defined scopes for the current form.
   */
  protected function getDefinedFormScopes(array $extra_fields = []) {
    // Pass the common field options relevant to this style.
    $fields = Defaults::viewFieldOptions();
    $fields = array_merge($fields, $extra_fields);

    // Fetches the returned field definitions to be used to define form scopes.
    $definition = $this->getDefinedFieldOptions($fields);
    $definition['opening_class'] = 'form--views';
    $definition['_views'] = TRUE;

    return $definition;
  }

  /**
   * {@inheritdoc}
   */
  protected function buildSettings() {
    $settings  = parent::buildSettings();
    $settings += Defaults::htmlSettings();
    $blazies   = $settings['blazies'];

    $blazies->set('namespace', 'gridstack')
      ->set('item.id', 'box')
      ->set('is.noratio', TRUE);

    // Prepare needed settings to work with.
    $settings['caption'] = array_filter($settings['caption']);
    $settings['ratio'] = '';

    return $settings;
  }

  /**
   * Build the Gridstack settings form.
   */
  protected function buildSettingsForm(&$form, &$definition) {
    $count = empty($definition['captions']) ? 0 : count($definition['captions']);
    $definition['captions_count'] = $count;
    $definition['stamps'] = $this->getViewsAsOptions('html_list');

    $this->admin()->buildSettingsForm($form, $definition);

    $title = '<p class="form__header form__title">';
    $title .= $this->t('Check Vanilla if using content/custom markups, not fields. <small>See it under <strong>Format > Show</strong> section. Otherwise Gridstack markups apply which require some fields added below.</small>');
    $title .= '</p>';

    $form['opening']['#markup'] .= $title;
    $form['image']['#description'] .= ' ' . $this->t('Be sure to UNCHECK "Use field template" (by default already UNCHECKED) to have it work for Blazy lazyloading. Use Blazy formatters for relevant features such as Colorbox/Photobox/Photoswipe, etc., or multimedia supports.');
  }

}
